import sys
from pathlib import Path
import os

try:
    import tomllib
except ImportError:
    # For Python < 3.11, you might need to pip install toml
    import toml as tomllib

# --- 基本路径 ---
PROJECT_ROOT = Path(os.getcwd())
"""项目根目录"""

CONFIG_FILE = PROJECT_ROOT / 'config.toml'

if not CONFIG_FILE.exists():
    CONFIG_FILE.write_text(
        """
# 配置文件示例
# 请根据实际情况修改以下内容
project_name = "NAME"
share_path = "\\\\192.168.a.b\\11\\22\\33"
"""
    )
    print(f'配置文件 {CONFIG_FILE} 不存在，已创建示例文件。请根据实际情况修改。')
    exit(1)

# --- 从 config.toml 加载配置 ---
try:
    with open(PROJECT_ROOT / 'config.toml', 'rb') as f:
        _config = tomllib.load(f)
    PROJECT_NAME = _config['project_name']
    SHARE_PATH = Path(_config['share_path'])
except (FileNotFoundError, KeyError) as e:
    print(f"错误: 无法加载或解析 'config.toml' 文件。")
    print(f"请确保该文件存在于 '{PROJECT_ROOT}' 目录下，")
    print(f"并且包含了 'project_name' 和 'share_path' 键。")
    print(f'详细错误: {e}')
    sys.exit(1)

# --- 派生路径和常量 ---
RELEASE_DIR = PROJECT_ROOT / 'releases'
"""存放最终发布版本和压缩包的目录"""

TEMP_DIR = RELEASE_DIR / 'temp'
"""用于构建过程的临时目录"""

DIST_DIR = PROJECT_ROOT / 'dist'
"""PyInstaller 的默认输出目录 (在此脚本中未使用，但作为参考)"""

BUILD_DIR = PROJECT_ROOT / 'build'
"""PyInstaller 的工作目录"""

SPEC_FILE = PROJECT_ROOT / f'{PROJECT_NAME}.spec'
"""PyInstaller 的 .spec 配置文件路径"""

if not SPEC_FILE.exists():
    SPEC_FILE.write_text(
        f"""# -*- mode: python ; coding: utf-8 -*-
a = Analysis(
    ['{PROJECT_NAME}.py'],
    pathex=[],
    binaries=[],
    datas=[
     ('.venv/Lib/site-packages/ddddocr/common_old.onnx', 'ddddocr'),
    ('.venv/Lib/site-packages/ddddocr/common.onnx', 'ddddocr'),
    ('.venv/Lib/site-packages/onnxruntime/capi/onnxruntime_providers_shared.dll', 'onnxruntime\capi'),
    ('.venv/Lib/site-packages/onnxruntime/capi/onnxruntime_pybind11_state.pyd', 'onnxruntime\capi'),
    ],
    hiddenimports=[],
    hookspath=[],
    hooksconfig={{}},
    runtime_hooks=[],
    excludes=[],
    noarchive=False,
    optimize=0,
)
pyz = PYZ(a.pure)

exe = EXE(
    pyz,
    a.scripts,
    a.binaries,
    a.datas,
    [],
    name='{PROJECT_NAME}',
    debug=False,
    bootloader_ignore_signals=False,
    strip=False,
    upx=True,
    upx_exclude=[],
    runtime_tmpdir=None,
    console=True,
    disable_windowed_traceback=False,
    argv_emulation=False,
    target_arch=None,
    codesign_identity=None,
    entitlements_file=None,
)
"""
    )
    print(f'未找到 {SPEC_FILE}，已创建默认的 PyInstaller 配置文件。')

VERSION_FILE = PROJECT_ROOT / 'VERSION'
"""存储版本号的文件"""

# --- 源目录 ---
文档目录 = PROJECT_ROOT / '文档'
浏览器配置文件 = PROJECT_ROOT / '配置文件'
浏览器 = PROJECT_ROOT / 'BIN'



from datetime import date
from . import config

def read_and_update_version():
    """
    读取并更新版本号。
    版本格式为 YYYY.M.D.rev。
    如果最后一次构建是今天，则修订号 (rev) 增加。
    如果最后一次构建不是今天，则修订号重置为 0。
    """
    today = date.today()

    # 默认新版本号（当天第一次构建）
    new_version_str = f'{today.year}.{today.month}.{today.day}.0'

    if config.VERSION_FILE.exists():
        last_version_str = config.VERSION_FILE.read_text(encoding='utf-8').strip()
        if last_version_str:
            try:
                parts = last_version_str.split('.')
                last_year, last_month, last_day, last_rev = map(int, parts)
                last_build_date = date(last_year, last_month, last_day)

                if last_build_date == today:
                    # 同一天构建，修订号递增
                    new_rev = last_rev + 1
                    new_version_str = f'{today.year}.{today.month}.{today.day}.{new_rev}'
                # else: 新的一天，使用上面已经设置好的默认版本号
            except (ValueError, IndexError):
                print(f"警告: VERSION 文件中的版本 '{last_version_str}' 格式无效。将从今天的日期重新开始。")
    
    # 写入新版本号
    config.VERSION_FILE.write_text(new_version_str, encoding='utf-8')
    print(f'版本号已更新为: {new_version_str}')
    return new_version_str


import subprocess
import shutil
import zipfile
import sys
from pathlib import Path
from tqdm import tqdm
from . import config

def build():
    """使用 PyInstaller 进行打包。"""
    print('1. 使用 PyInstaller 打包...')
    subprocess.run(
        [
            sys.executable,
            '-m',
            'PyInstaller',
            '--clean',
            '--distpath',
            str(config.TEMP_DIR),
            '--workpath',
            str(config.BUILD_DIR),
            str(config.SPEC_FILE),
        ],
        check=True,
    )

def copy_dirs():
    """复制必要的目录（浏览器、配置文件、文档）到临时构建目录。"""
    print('2. 复制目录到 release...')
    for d in [config.浏览器, config.浏览器配置文件, config.文档目录]:
        if d.exists():
            shutil.copytree(d, config.TEMP_DIR / d.name, dirs_exist_ok=True)

    bin_dir = config.TEMP_DIR / 'phis_bin'
    # rename it to BIN
    if bin_dir.exists():
        bin_dir.rename(config.TEMP_DIR / 'BIN')

    # 移动 env.txt 到特定的子目录
    执行结果目录 = config.TEMP_DIR / '执行结果'
    执行结果目录.mkdir(exist_ok=True)
    env_file_source = config.TEMP_DIR / '文档' / 'env.txt'
    if env_file_source.exists():
        shutil.move(str(env_file_source), 执行结果目录 / 'env.txt')

def copy_to_release_dir(version: str) -> Path:
    """将临时目录的内容复制到带版本号的最终发布目录。"""
    target_dir = config.RELEASE_DIR / f'{config.PROJECT_NAME}_v{version}'
    if target_dir.exists():
        shutil.rmtree(target_dir)
    shutil.copytree(config.TEMP_DIR, target_dir, dirs_exist_ok=True)
    return target_dir

def make_zip(target_dir: Path, version: str) -> Path:
    """将发布目录压缩成 zip 文件。"""
    zip_path = target_dir.parent / f'{config.PROJECT_NAME}_v{version}.zip'
    print(f'3. 压缩为 {zip_path} ...')
    with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as zf:
        for file in target_dir.rglob('*'):
            zf.write(file, file.relative_to(target_dir.parent))
    print(f'已创建压缩包: {zip_path}')
    return zip_path

def copy_to_share(file: Path):
    """尝试将文件复制到网络共享位置，如果失败则忽略。"""
    try:
        destination_file = config.SHARE_PATH / file.name
        print(f'4. 尝试复制到共享目录 {destination_file} ...')

        config.SHARE_PATH.mkdir(parents=True, exist_ok=True)
        file_size = file.stat().st_size

        with (
            open(file, 'rb') as fsrc,
            open(destination_file, 'wb') as fdst,
            tqdm(total=file_size, unit='B', unit_scale=True, desc=f'复制 {file.name}') as pbar,
        ):
            while True:
                chunk = fsrc.read(4096 * 1024)  # 4MB chunks
                if not chunk:
                    break
                fdst.write(chunk)
                pbar.update(len(chunk))

        print(f'\n成功复制到: {destination_file}')
    except Exception as e:
        print(f'\n警告: 复制到共享目录失败，已忽略。错误: {e}')

def copy_dir_to_share(source_dir: Path):
    """尝试将整个目录复制到网络共享位置。"""
    try:
        destination_dir = config.SHARE_PATH / source_dir.name
        print(f'4. 尝试将目录复制到共享位置 {destination_dir} ...')

        if destination_dir.exists():
            print(f"警告: 目标目录 {destination_dir} 已存在。正在删除旧目录...")
            shutil.rmtree(destination_dir)

        # 使用 shutil.copytree 复制整个目录
        shutil.copytree(source_dir, destination_dir, dirs_exist_ok=True)

        print(f'\n成功将目录复制到: {destination_dir}')
    except Exception as e:
        print(f'\n警告: 复制目录到共享位置失败，已忽略。错误: {e}')

def clean_temp_dir():
    """清理临时构建目录。"""
    if config.TEMP_DIR.exists():
        shutil.rmtree(config.TEMP_DIR)